@@ -437,7 +437,7 @@ forecast_seasvary <- function(n_seas,
 
 
 ## HAS_TESTS
-#' Create Extra Rows for 'data', holding Values for
+#' Create Extra Rows for 'data' that Hold Values for
 #' Predictor Variables Used in Forecast
 #'
 #' @param mod Object of class 'bage_mod'
@@ -451,17 +451,60 @@ make_data_forecast_labels <- function(mod, labels_forecast) {
   formula <- mod$formula
   data <- mod$data
   var_time <- mod$var_time
-  nms_model <- all.vars(formula[-2L])
-  ans <- lapply(data[nms_model], unique)
+  vars <- all.vars(formula[-2L])
+  ans <- lapply(data[vars], unique)
   ans[[var_time]] <- labels_forecast
   ans <- vctrs::vec_expand_grid(!!!ans)
   ans <- vctrs::vec_rbind(data, ans)
   i_original <- seq_len(nrow(data))
   ans <- ans[-i_original, , drop = FALSE]
+  if (has_covariates(mod))
+    ans <- make_data_forecast_labels_covariates(mod = mod,
+                                                data_forecast = ans)
   ans
 }
 
 
+## HAS_TESTS
+#' Cbind Covariate Values to 'data_forecast' Made From Labels
+#'
+#' Used with 'make_data_forecast_labels()'
+#'
+#' @param mod Object of class 'bage_mod'
+#' @param data_forecast Data for forecast, without covariates
+#' (ie classifying variables only)
+#'
+#' @returns A tibble.
+#'
+#' @noRd
+make_data_forecast_labels_covariates <- function(mod, data_forecast) {
+  data <- mod$data
+  formula <- mod$formula
+  formula_covariates <- mod$formula_covariates
+  var_time <- mod$var_time
+  vars <- all.vars(formula[-2L])
+  vars_covariates <- all.vars(formula_covariates)
+  vars_no_time <- setdiff(vars, var_time)
+  map_vars_cov <- vctrs::vec_split(data[vars_covariates], data[vars_no_time])
+  map_vars_cov$val <- lapply(map_vars_cov$val, unique)
+  nrow <- vapply(map_vars_cov$val, nrow, 1L)
+  i_gt_1 <- match(TRUE, nrow > 1L, nomatch = 0L)
+  if (i_gt_1 > 0L) {
+    cli::cli_abort(c("Unable to derive covariate values for forecasted periods.",
+                     i = paste("Can't predict values for covariates",
+                               "based only on {.var {vars_no_time}}.")))
+  }
+  key_data <- Reduce(paste_dot, data_forecast[vars_no_time])
+  key_covariates <- Reduce(paste_dot, map_vars_cov$key)
+  covariates <- map_vars_cov$val[match(key_data, key_covariates)]
+  covariates <- vctrs::vec_rbind(!!!covariates)
+  ans <- tibble::tibble(data_forecast)
+  for (nm in names(covariates))
+    ans[[nm]] <- covariates[[nm]]
+  ans
+}
+  
+
 ## HAS_TESTS
 #' Construct 'data_forecast' from 'newdata' Argument to 'forecast'
 #'
@@ -479,6 +522,10 @@ make_data_forecast_newdata <- function(mod, newdata) {
   data <- mod$data
   var_time <- mod$var_time
   nms_model <- all.vars(formula[-2L])
+  if (has_covariates(mod)) {
+    formula_covariates <- mod$formula_covariates
+    nms_model <- c(nms_model, all.vars(formula_covariates))
+  }
   nms_newdata <- names(newdata)
   not_in_newdata <- !(nms_model %in% nms_newdata)
   n_not_in_newdata <- sum(not_in_newdata)
@@ -487,13 +534,17 @@ make_data_forecast_newdata <- function(mod, newdata) {
                          "{.val {nms_model[not_in_newdata]}}."))
   labels_new <- unique(newdata[[var_time]])
   labels_est <- unique(data[[var_time]])
-  duplicates <- intersect(labels_new, labels_est)
-  n_dup <- length(duplicates)
-  if (n_dup > 0L)
-    cli::cli_abort(c("Time periods in {.arg newdata} and {.arg data} overlap.",
-                     i = "Time periods in {.arg newdata}: {.val {labels_new}}.",
-                     i = "Time periods in {.arg data}: {.val {labels_est}}.",
+  i_dup <- match(labels_new, labels_est, nomatch = 0L)
+  n_dup <- sum(i_dup > 0L)
+  if (n_dup > 0L) {
+    duplicates <- labels_est[i_dup]
+    if (inherits(duplicates, "Date"))
+      duplicates <- format(duplicates, "%Y-%m-%d")
+    cli::cli_abort(c("Times in {.arg newdata} and {.arg data} overlap.",
+                     i = "Times in {.arg newdata}: {.val {labels_new}}.",
+                     i = "Times in {.arg data}: {.val {labels_est}}.",
                      i = "Overlap: {.val {duplicates}}."))
+  }
   ans <- vctrs::vec_rbind(data, newdata)
   i_original <- seq_len(nrow(data))
   ans <- ans[-i_original, , drop = FALSE]
