@@ -1,4 +1,131 @@
 
+## 'set_seeds' --------------------------------------------------------------
+
+test_that("'set_seeds' works with NULL", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$deaths <- rpois(n = nrow(data), lambda = 3)
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age:sex + time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn) |>
+    fit()
+  aug1 <- augment(mod)
+  aug2 <- augment(mod)
+  expect_identical(aug1, aug2)
+  comp1 <- components(mod)
+  comp2 <- components(mod)
+  expect_identical(comp1, comp2)
+  faug1 <- forecast(mod, labels = 2002)
+  faug2 <- forecast(mod, labels = 2002)
+  expect_identical(faug1, faug2)
+  comp1 <- components(mod)
+  comp2 <- components(mod)
+  expect_identical(comp1, comp2)
+  fcomp1 <- forecast(mod, labels = 2002, output = "comp")
+  fcomp2 <- forecast(mod, labels = 2002, output = "comp")
+  expect_identical(fcomp1, fcomp2)
+  mod <- set_seeds(mod)
+  expect_false(is_fitted(mod))
+  mod <- fit(mod)
+  aug3 <- augment(mod)
+  expect_false(identical(aug1, aug3))
+  comp3 <- components(mod)
+  expect_false(identical(comp1, comp3))
+  comp4 <- components(mod)
+  expect_identical(comp3, comp4)
+  faug3 <- forecast(mod, labels = 2002)
+  expect_false(identical(faug1, faug3))
+  fcomp3 <- forecast(mod, labels = 2002, output = "comp")
+  expect_false(identical(fcomp1, fcomp3))
+  fcomp4 <- forecast(mod, labels = 2002, output = "comp")
+  expect_identical(fcomp3, fcomp4)
+})
+
+test_that("'set_seeds' works with list", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$deaths <- rpois(n = nrow(data), lambda = 3)
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age:sex + time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn) |>
+    fit()
+  aug1 <- augment(mod)
+  aug2 <- augment(mod)
+  expect_identical(aug1, aug2)
+  comp1 <- components(mod)
+  comp2 <- components(mod)
+  expect_identical(comp1, comp2)
+  mod <- set_seeds(mod,
+                     new_seeds = list(seed_components = 1,
+                                      seed_augment = 2,
+                                      seed_forecast_components = 3,
+                                      seed_forecast_augment = 4))
+  expect_false(is_fitted(mod))
+  mod <- fit(mod)
+  aug3 <- augment(mod)
+  expect_false(identical(aug1, aug3))
+  comp3 <- components(mod)
+  expect_false(identical(comp1, comp3))
+  expect_identical(mod$seed_components, 1)
+  expect_identical(mod$seed_augment, 2)
+  expect_identical(mod$seed_forecast_components, 3)
+  expect_identical(mod$seed_forecast_augment, 4)
+})
+
+
+## 'set_covariates' -----------------------------------------------------------
+
+test_that("'set_covariates' works with Poisson", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$deaths <- rpois(n = nrow(data), lambda = 3)
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age:sex + time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn)
+  ans_obtained <- set_covariates(mod, ~ income)
+  ans_expected <- mod
+  ans_expected$formula_covariates <- ~ income
+  ans_expected$covariates_nms <- "income"
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'set_covariates' works with binomial", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$deaths <- rbinom(n = nrow(data), size = data$popn, prob = 0.3)
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age:sex + time
+  mod <- mod_binom(formula = formula,
+                   data = data,
+                   size = popn)
+  ans_obtained <- set_covariates(mod, ~ income)
+  ans_expected <- mod
+  ans_expected$formula_covariates <- ~income
+  ans_expected$covariates_nms <- "income"
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'set_covariates' issues message with existing covariates", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$deaths <- rpois(n = nrow(data), lambda = 3)
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age:sex + time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn) |>
+    set_covariates(~ income)
+  expect_message(set_covariates(mod, ~ income),
+                "Model already has covariates\\. Deleting these\\.")
+})
+
+
 ## 'set_datamod_outcome_rr3' --------------------------------------------------
 
 test_that("'set_datamod_outcome_rr3' works with Poisson", {
@@ -85,6 +212,8 @@ test_that("'set_disp' works with Poisson", {
     expect_identical(mod$mean_disp, 0)
     mod <- set_disp(mod, mean = 0.5)
     expect_identical(mod$mean_disp, 0.5)
+    mod <- set_disp(mod)
+    expect_identical(mod$mean_disp, 1)
 })
 
 test_that("'set_disp' works with normal", {
@@ -98,6 +227,8 @@ test_that("'set_disp' works with normal", {
     mod <- set_disp(mod, mean = 0.5)
     expect_identical(mod$mean_disp, 0.5)
     expect_error(set_disp(mod, mean = 0))
+    mod <- set_disp(mod)
+    expect_identical(mod$mean_disp, 1)
 })
 
 
@@ -121,6 +252,31 @@ test_that("'set_n_draw' works with valid inputs", {
                  "New value")
 })
 
+test_that("'set_n_draw' reduces draws in fitted object", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$deaths <- rev(seq_len(nrow(data)))
+  data$income <- rnorm(nrow(data))
+  formula <- deaths ~ age:sex + time
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn)
+  mod <- set_prior(mod, time ~ Lin_AR1())
+  mod <- set_covariates(mod, ~ income)
+  mod <- set_n_draw(mod, n_draw = 10L)
+  mod <- fit(mod)
+  expect_identical(ncol(mod$draws_effectfree), 10L)
+  expect_identical(ncol(mod$draws_hyper), 10L)
+  expect_identical(ncol(mod$draws_hyperrandfree), 10L)
+  expect_identical(ncol(mod$draws_coef_covariates), 10L)
+  mod <- set_n_draw(mod, n_draw = 5)
+  expect_identical(mod$n_draw, 5L)
+  expect_identical(ncol(mod$draws_effectfree), 5L)
+  expect_identical(ncol(mod$draws_hyper), 5L)
+  expect_identical(ncol(mod$draws_hyperrandfree), 5L)
+  expect_identical(ncol(mod$draws_coef_covariates), 5L)
+})
+
 
 ## 'set_prior' ----------------------------------------------------------------
 
@@ -334,31 +490,35 @@ test_that("'set_var_inner' gives correct errors with invalid inputs", {
 
 ## 'unfit' --------------------------------------------------------------------
 
-test_that("'set_n_draw' works with valid inputs", {
-    data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
-    data$popn <- seq_len(nrow(data))
-    data$deaths <- rev(seq_len(nrow(data)))
-    formula <- deaths ~ age:sex + time
-    mod_unfit <- mod_pois(formula = formula,
-                    data = data,
-                    exposure = popn)
-    mod_fit <- fit(mod_unfit)
-    mod_fit_unfit <- unfit(mod_unfit)
-    nms <- c("draws_effectfree",
-             "draws_hyper",
-             "draws_hyperrandfree",
-             "draws_disp",
-             "point_effectfree",
-             "point_hyper",
-             "point_hyperrandfree",
-             "point_disp",
-             "computations",
-             "oldpar")
-    expect_true(all(nms %in% names(mod_unfit)))
-    expect_true(all(nms %in% names(mod_fit)))
-    expect_true(all(nms %in% names(mod_fit_unfit)))
-    for (nm in nms)
-      expect_false(isTRUE(all.equal(mod_fit[[nm]], mod_unfit[[nm]])))
-    for (nm in nms)
-      expect_true(isTRUE(all.equal(mod_fit_unfit[[nm]], mod_unfit[[nm]])))
+test_that("'unfit' works with valid inputs", {
+  data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
+  data$popn <- seq_len(nrow(data))
+  data$deaths <- rev(seq_len(nrow(data)))
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age:sex + time
+  mod_unfit <- mod_pois(formula = formula,
+                        data = data,
+                        exposure = popn) |>
+    set_covariates(~ income)
+  mod_fit <- fit(mod_unfit)
+  mod_fit_unfit <- unfit(mod_unfit)
+  nms <- c("draws_effectfree",
+           "draws_hyper",
+           "draws_hyperrandfree",
+           "draws_coef_covariates",
+           "draws_disp",
+           "point_effectfree",
+           "point_hyper",
+           "point_hyperrandfree",
+           "point_coef_covariates",
+           "point_disp",
+           "computations",
+           "oldpar")
+  expect_true(all(nms %in% names(mod_unfit)))
+  expect_true(all(nms %in% names(mod_fit)))
+  expect_true(all(nms %in% names(mod_fit_unfit)))
+  for (nm in nms)
+    expect_false(isTRUE(all.equal(mod_fit[[nm]], mod_unfit[[nm]])))
+  for (nm in nms)
+    expect_true(isTRUE(all.equal(mod_fit_unfit[[nm]], mod_unfit[[nm]])))
 })
