@@ -159,26 +159,26 @@ test_that("'dimnames_to_nm_split' works with 2D dimnames", {
 ## 'eval_offset_formula' ------------------------------------------------------
 
 test_that("'eval_offset_formula' works with valid inputs - simple formula", {
-  vname_offset <- "~popn + other"
+  nm_offset_data <- "~popn + other"
   data <- data.frame(popn = 1, other = 2)
-  ans_obtained <- eval_offset_formula(vname_offset = vname_offset, data = data)
+  ans_obtained <- eval_offset_formula(nm_offset_data = nm_offset_data, data = data)
   ans_expected <- 3
   expect_identical(ans_obtained, ans_expected)
 })
 
 test_that("'eval_offset_formula' works with valid inputs - complicated formula", {
-  vname_offset <- "~popn^2 + log(other) + 6"
+  nm_offset_data <- "~popn^2 + log(other) + 6"
   data <- data.frame(popn = 1:2, other = 2:3)
-  ans_obtained <- eval_offset_formula(vname_offset = vname_offset, data = data)
+  ans_obtained <- eval_offset_formula(nm_offset_data = nm_offset_data, data = data)
   ans_expected <- (1:2)^2 + log(2:3) + 6
   expect_identical(ans_obtained, ans_expected)
 })
 
 
 test_that("'eval_offset_formula' works with valid inputs - ifelse", {
-  vname_offset <- "~ifelse(popn <= 0, 0.1, popn)"
+  nm_offset_data <- "~ifelse(popn <= 0, 0.1, popn)"
   data <- data.frame(popn = 0:2)
-  ans_obtained <- eval_offset_formula(vname_offset = vname_offset, data = data)
+  ans_obtained <- eval_offset_formula(nm_offset_data = nm_offset_data, data = data)
   ans_expected <- c(0.1, 1, 2)
   expect_identical(ans_obtained, ans_expected)
 })
@@ -349,6 +349,149 @@ test_that("'infer_var_time' returns NULL when not single valid answer", {
 })
 
 
+## 'get_is_in_lik' ----------------------------------------------------------------
+
+test_that("'get_is_in_lik' works with no NAs", {
+    set.seed(0)
+    data <- expand.grid(age = 0:2,
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    data$income <- rnorm(n = nrow(data))
+    formula <- deaths ~ age + sex
+    mod <- mod_pois(formula = formula,
+                    data = data,
+                    exposure = popn)
+    ans_obtained <- get_is_in_lik(mod)
+    ans_expected <- rep(TRUE, 6)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'get_is_in_lik' works with NAs", {
+    set.seed(0)
+    data <- expand.grid(age = c(0:1, NA),
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    data$popn[1] <- 0
+    data$deaths[1] <- 0
+    formula <- deaths ~ age + sex
+    mod <- mod_pois(formula = formula,
+                    data = data,
+                    exposure = popn)
+    ans_obtained <- get_is_in_lik(mod)
+    ans_expected <- c(FALSE, TRUE, FALSE, TRUE, TRUE, FALSE)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+
+## 'get_is_in_lik_covariates' --------------------------------------------------------
+
+test_that("'get_is_in_lik_covariates' works with no NAs", {
+    set.seed(0)
+    data <- expand.grid(age = 0:2,
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    data$income <- rnorm(n = nrow(data))
+    formula <- deaths ~ age + sex
+    mod <- mod_pois(formula = formula,
+                    data = data,
+                    exposure = popn) |>
+      set_covariates(~ income)
+    ans_obtained <- get_is_in_lik_effects(mod)
+    ans_expected <- rep(TRUE, 6)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'get_is_in_lik_effects' works with NAs", {
+    set.seed(0)
+    data <- expand.grid(age = c(0:1, NA),
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    formula <- deaths ~ age + sex
+    mod <- mod_pois(formula = formula,
+                    data = data,
+                    exposure = popn)
+    ans_obtained <- get_is_in_lik_effects(mod)
+    ans_expected <- rep(c(TRUE, TRUE, FALSE), 2)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+
+## 'get_is_in_lik_effects' --------------------------------------------------------
+
+test_that("'get_is_in_lik_effects' works with no NAs", {
+    set.seed(0)
+    data <- expand.grid(age = 0:2,
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    formula <- deaths ~ age + sex
+    mod <- mod_pois(formula = formula,
+                    data = data,
+                    exposure = popn)
+    ans_obtained <- get_is_in_lik_effects(mod)
+    ans_expected <- rep(TRUE, 6)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'get_is_in_lik_effects' works with NAs", {
+    set.seed(0)
+    data <- expand.grid(age = c(0:1, NA),
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    formula <- deaths ~ age + sex
+    mod <- mod_pois(formula = formula,
+                    data = data,
+                    exposure = popn)
+    ans_obtained <- get_is_in_lik_effects(mod)
+    ans_expected <- rep(c(TRUE, TRUE, FALSE), 2)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+
+## 'get_is_in_lik_offset' -----------------------------------------------------
+
+test_that("'get_is_in_lik_offset' works with no NAs", {
+    mod <- list(outcome = c(0, 1, 5),
+                offset = c(1, 0, 3))
+    ans_obtained <- get_is_in_lik_offset(mod)
+    ans_expected <- c(TRUE, FALSE, TRUE)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'get_is_in_lik_offset' works with NAs", {
+    mod <- list(outcome = c(0, 1, NA, 7),
+                offset = c(1, 0, 3, NA))
+    ans_obtained <- get_is_in_lik_offset(mod)
+    ans_expected <- c(TRUE, FALSE, TRUE, FALSE)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+
+## 'get_is_in_lik_outcome' ----------------------------------------------------
+
+test_that("'get_is_in_lik_outcome' works with no NAs", {
+    mod <- list(outcome = c(0, 1, 5),
+                offset = c(1, 0, 3))
+    ans_obtained <- get_is_in_lik_outcome(mod)
+    ans_expected <- c(TRUE, TRUE, TRUE)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'get_is_in_lik_outcome' works with NAs", {
+    mod <- list(outcome = c(0, 1, NA, 7),
+                offset = c(1, 0, 3, NA))
+    ans_obtained <- get_is_in_lik_outcome(mod)
+    ans_expected <- c(TRUE, TRUE, FALSE, TRUE)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+
+
 ## 'make_agesex' --------------------------------------------------------------
 
 test_that("'make_agesex' works with valid inputs", {
@@ -403,6 +546,30 @@ test_that("'make_agesex' works with valid inputs", {
 })
 
 
+## 'make_coef_covariates' -----------------------------------------------------
+
+test_that("'make_coef_covariates' works", {
+    set.seed(0)
+    data <- expand.grid(age = 0:9,
+                        region = c("a", "b"),
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    data$income <- runif(n = nrow(data))
+    data$distance <- runif(n = nrow(data))
+    mod <- mod_pois(formula = deaths ~ age * sex,
+                    data = data,
+                    exposure = popn)
+    ans_obtained <- make_coef_covariates(mod)
+    ans_expected <- double()
+    expect_identical(ans_obtained, ans_expected)
+    mod <- set_covariates(mod, ~ income + distance)
+    ans_obtained <- make_coef_covariates(mod)
+    ans_expected <- c(income = 0, distance = 0)
+    expect_identical(ans_obtained, ans_expected)
+})
+
+
 ## 'make_const' --------------------------------------------------------------- 
 
 test_that("'make_const' works with valid inputs", {
@@ -441,6 +608,28 @@ test_that("'make_const' works with valid inputs - no terms", {
 })
 
 
+## 'make_data_df' -------------------------------------------------------------
+
+test_that("'make_data_df' works", {
+    set.seed(0)
+    data <- expand.grid(age = 0:9,
+                        time = 2000:2005,
+                        sex = c("F", "M"))
+    data$popn <- rpois(n = nrow(data), lambda = 100)
+    data$deaths <- rpois(n = nrow(data), lambda = 10)
+    data$deaths[1] <- NA
+    mod <- mod_pois(deaths ~ age * sex + time,
+                    data = data,
+                    exposure = popn)
+    ans_obtained <- make_data_df(mod)
+    ans_expected <- data[-1,]
+    ans_expected$deaths <- as.double(ans_expected$deaths)
+    ans_expected$popn <- as.double(ans_expected$popn)
+    ans_expected <- tibble::tibble(ans_expected)
+    expect_identical(ans_obtained, ans_expected)    
+})
+
+
 ## 'make_dimnames_terms' ------------------------------------------------------
 
 test_that("'make_dimnames_terms' works - includes intercept", {
@@ -477,7 +666,7 @@ test_that("'make_dimnames_terms' works - no intercept", {
 })
 
 
-## 'make_effectfree' -------------------------------------------------------------
+## 'make_effectfree' ----------------------------------------------------------
 
 test_that("'make_effectfree' works with valid inputs", {
     set.seed(0)
@@ -578,26 +767,7 @@ test_that("'make_i_prior' works with valid inputs", {
 })
 
 
-## 'make_is_in_lik' -----------------------------------------------------------
-
-test_that("'make_is_in_lik' works with no NAs", {
-    mod <- list(outcome = c(0, 1, 5),
-                offset = c(1, 0, 3))
-    ans_obtained <- make_is_in_lik(mod)
-    ans_expected <- c(TRUE, FALSE, TRUE)
-    expect_identical(ans_obtained, ans_expected)
-})
-
-test_that("'make_is_in_lik' works with NAs", {
-    mod <- list(outcome = c(0, 1, NA, 7),
-                offset = c(1, 0, 3, NA))
-    ans_obtained <- make_is_in_lik(mod)
-    ans_expected <- c(TRUE, FALSE, FALSE, FALSE)
-    expect_identical(ans_obtained, ans_expected)
-})
-
-
-## 'make_lengths_effect' ---------------------------------------------------------
+## 'make_lengths_effect' ------------------------------------------------------
 
 test_that("'make_lengths_effect' works with valid inputs - has intercept", {
   set.seed(0)
@@ -635,7 +805,7 @@ test_that("'make_lengths_effect' works with valid inputs - no intercept", {
 })
 
 
-## 'make_lengths_effectfree' -----------------------------------------------------------
+## 'make_lengths_effectfree' --------------------------------------------------
 
 test_that("'make_lengths_effectfree' works with valid inputs", {
     set.seed(0)
@@ -830,101 +1000,6 @@ test_that("'make_levels_forecast_all' works with no intercept", {
 })
 
 
-## 'make_map' -----------------------------------------------------------------
-
-test_that("'make_map' works with no parameters fixed", {
-    set.seed(0)
-    data <- expand.grid(time = 2000:2009,
-                        region = 1:2,
-                        SEX = c("F", "M"))
-    data$popn <- rpois(n = nrow(data), lambda = 100)
-    data$deaths <- rpois(n = nrow(data), lambda = 10)
-    formula <- deaths ~ time * SEX + region
-    mod <- mod_pois(formula = formula,
-                    data = data,
-                    exposure = popn)
-    ans_obtained <- make_map(mod)
-    ans_expected <- NULL
-    expect_identical(ans_obtained, ans_expected)
-})
-
-test_that("'make_map' works when 'effectfree' contains known values", {
-    set.seed(0)
-    data <- expand.grid(time = 0:3,
-                        SEX = c("F", "M"))
-    data$popn <- rpois(n = nrow(data), lambda = 100)
-    data$deaths <- rpois(n = nrow(data), lambda = 10)
-    formula <- deaths ~ time * SEX
-    mod <- mod_pois(formula = formula,
-                    data = data,
-                    exposure = popn)
-    mod <- set_prior(mod, SEX ~ Known(c(0.1, -0.1)))
-    ans_obtained <- make_map(mod)
-    ans_expected <- list(effectfree = factor(c("(Intercept)" = 1,
-                                            time = 2,
-                                            time = 3,
-                                            time = 4,
-                                            time = 5,
-                                            SEX = NA,
-                                            SEX = NA,
-                                            "time:SEX" = 6,
-                                            "time:SEX" = 7,
-                                            "time:SEX" = 8,
-                                            "time:SEX" = 9,
-                                            "time:SEX" = 10,
-                                            "time:SEX" = 11,
-                                            "time:SEX" = 12,
-                                            "time:SEX" = 13)))
-    expect_identical(ans_obtained, ans_expected)
-})
-
-test_that("'make_map' works dispersion is 0", {
-    set.seed(0)
-    data <- expand.grid(time = 2000:2009,
-                        region = 1:2,
-                        SEX = c("F", "M"))
-    data$popn <- rpois(n = nrow(data), lambda = 100)
-    data$deaths <- rpois(n = nrow(data), lambda = 10)
-    formula <- deaths ~ time * SEX + region
-    mod <- mod_pois(formula = formula,
-                    data = data,
-                    exposure = popn)
-    mod <- set_disp(mod, mean = 0)
-    ans_obtained <- make_map(mod)
-    ans_expected <- list(log_disp = factor(NA))
-    expect_identical(ans_obtained, ans_expected)
-})
-
-test_that("'make_map' works when effectfree has known values", {
-    set.seed(0)
-    data <- expand.grid(time = 0:3,
-                        SEX = c("F", "M"))
-    data$popn <- rpois(n = nrow(data), lambda = 100)
-    data$deaths <- rpois(n = nrow(data), lambda = 10)
-    formula <- deaths ~ time * SEX
-    mod <- mod_pois(formula = formula,
-                    data = data,
-                    exposure = popn) |>
-      set_prior(time ~ RW(sd = 0)) |>
-      set_prior(time:SEX ~ RW(sd = 0))
-    mod <- set_prior(mod, SEX ~ Known(c(0.1, -0.1)))
-    ans_obtained <- make_map(mod)
-    ans_expected <- list(effectfree = factor(c("(Intercept)" = 1,
-                                            time = 2,
-                                            time = 3,
-                                            time = 4,
-                                            SEX = NA,
-                                            SEX = NA,
-                                            "time:SEX" = 5,
-                                            "time:SEX" = 6,
-                                            "time:SEX" = 7,
-                                            "time:SEX" = 8,
-                                            "time:SEX" = 9,
-                                            "time:SEX" = 10)))
-    expect_identical(ans_obtained, ans_expected)
-})
-
-
 ## 'make_map_effectfree_fixed' ---------------------------------------------------
 
 test_that("'make_map_effectfree_fixed' works with valid inputs", {
@@ -1110,12 +1185,70 @@ test_that("'make_matrices_effectfree_effect' works with valid inputs", {
 })
 
 
+## 'make_matrix_covariates' ---------------------------------------------------
+
+test_that("'make_matrix_covariates' works with valid inputs - all numeric", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = c("a", "b"),
+                      sex = c("F", "M"))
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$income <- runif(n = nrow(data))
+  data$distance <- runif(n = nrow(data))
+  formula <- ~ income + distance
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data)
+  ans_expected <- model.matrix(~income + distance - 1, data = data)
+  ans_expected[,"income"] <- scale(ans_expected[,"income"])
+  ans_expected[,"distance"] <- scale(ans_expected[,"distance"])
+  rownames(ans_expected) <- NULL
+  attributes(ans_expected)$assign <- NULL
+  rownames(ans_expected) <- NULL
+  expect_equal(ans_obtained, ans_expected)
+})
+
+test_that("'make_matrix_covariates' works with valid inputs - not all numeric - has intercept", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = c("a", "b"),
+                      sex = c("F", "M"))
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$income <- runif(n = nrow(data))
+  formula <- ~ income * region
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data)
+  data_scaled <- data
+  data_scaled$income <- as.numeric(scale(data_scaled$income))
+  ans_expected <- model.matrix(~income*region, data = data_scaled)[,-1]
+  rownames(ans_expected) <- NULL
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'make_matrix_covariates' works with valid inputs - not all numeric - no intercept", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = c("a", "b"),
+                      sex = c("F", "M"))
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$income <- runif(n = nrow(data))
+  formula <- ~ income * region - 1
+  ans_obtained <- make_matrix_covariates(formula = formula, data = data)
+  data_scaled <- data
+  data_scaled$income <- as.numeric(scale(data_scaled$income))
+  ans_expected <- model.matrix(~income*region, data = data_scaled)[,-1]
+  attr(ans_expected, "assign") <- NULL
+  rownames(ans_expected) <- NULL
+  expect_identical(ans_obtained, ans_expected)
+})
+
+
 ## 'make_offset' --------------------------------------------------------------
 
 test_that("'make_offset' works with valid inputs - no NA", {
     data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
     data$wt <- seq_len(nrow(data))
-    ans_obtained <- make_offset(vname_offset = "wt",
+    ans_obtained <- make_offset(nm_offset_data = "wt",
                                 data = data)
     ans_expected <- as.double(data$wt)
     expect_identical(ans_obtained, ans_expected)
@@ -1125,7 +1258,7 @@ test_that("'make_offset' works with valid inputs - has NA", {
     data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
     data$wt <- seq_len(nrow(data))
     data$wt[3] <- NA
-    ans_obtained <- make_offset(vname_offset = "wt",
+    ans_obtained <- make_offset(nm_offset_data = "wt",
                                 data = data)
     ans_expected <- xtabs(wt ~ age + sex + time, data = data)
     ans_expected[3] <- NA
@@ -1136,7 +1269,7 @@ test_that("'make_offset' works with valid inputs - has NA", {
 test_that("'make_offset' works with valid inputs - no NA", {
     data <- expand.grid(age = 0:2, time = 2000:2001, sex = 1:2)
     data$wt <- seq_len(nrow(data))
-    ans_obtained <- make_offset(vname_offset = "~ wt + 1",
+    ans_obtained <- make_offset(nm_offset_data = "~ wt + 1",
                                 data = data)
     ans_expected <- as.double(data$wt) + 1
     expect_identical(ans_obtained, ans_expected)
@@ -1191,6 +1324,155 @@ test_that("'make_outcome' works with valid inputs", {
 })
 
 
+## 'make_outcome_offset_matrices' ---------------------------------------------
+
+test_that("'make_outcome_offset_matrices' works with model with offset", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = 1:2,
+                      sex = c("F", "M"),
+                      time = 1:2)
+  data$age[c(1, 41)] <- NA
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  formula <- deaths ~ age * sex + region
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn)
+  ans_obtained <- make_outcome_offset_matrices(mod, aggregate = TRUE)
+  data_ag <- aggregate(data[c("deaths", "popn")], data[c("age", "region", "sex")], sum)
+  data_ag <- data_ag[with(data_ag, order(age, sex, region)), ]
+  ans_expected <- list(outcome = data_ag[["deaths"]],
+                       offset = data_ag[["popn"]],
+                       matrices_effect_outcome = make_matrices_effect_outcome(data_ag,
+                                                                              mod$dimnames_terms),
+                       matrix_covariates = matrix(NA_real_, nrow = 0, ncol = 0))
+  expect_equal(ans_obtained, ans_expected)
+})
+
+test_that("'make_outcome_offset_matrices' works with model without offset", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = 1:2,
+                      sex = c("F", "M"),
+                      time = 1:2)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  formula <- deaths ~ age * sex + region
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = 1)
+  ans_obtained <- make_outcome_offset_matrices(mod, aggregate = TRUE)
+  data_ag <- aggregate(data["deaths"], data[c("age", "sex", "region")], sum)
+  ans_expected <- list(outcome = data_ag[["deaths"]],
+                       offset = rep(1, times = nrow(data_ag)),
+                       matrices_effect_outcome = make_matrices_effect_outcome(data_ag,
+                                                                              mod$dimnames_terms),
+                       matrix_covariates = matrix(NA_real_, nrow = 0, ncol = 0))
+  expect_equal(ans_obtained, ans_expected)
+})
+
+test_that("'make_outcome_offset_matrices' works with model with offset", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = 1:2,
+                      sex = c("F", "M"),
+                      time = 1:2)
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$popn[1] <- 0
+  data$deaths[1] <- 0
+  formula <- deaths ~ age * sex + region
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn)
+  ans_obtained <- make_outcome_offset_matrices(mod, aggregate = FALSE)
+  ans_expected <- list(outcome = mod$outcome[-1],
+                       offset = mod$offset[-1],
+                       matrices_effect_outcome = make_matrices_effect_outcome(data[-1,],
+                                                                              mod$dimnames_terms),
+                       matrix_covariates = matrix(NA_real_, nrow = 0, ncol = 0))
+  expect_equal(ans_obtained, ans_expected)
+})
+
+test_that("'make_outcome_offset_matrices' works with model with offset", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = 1:2,
+                      sex = c("F", "M"),
+                      time = 1:2)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$deaths[1] <- NA
+  formula <- deaths ~ age * sex + region
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = 1)
+  ans_obtained <- make_outcome_offset_matrices(mod, aggregate = FALSE)
+  ans_expected <- list(outcome = mod$outcome[-1],
+                       offset = mod$offset[-1],
+                       matrices_effect_outcome = make_matrices_effect_outcome(data[-1,],
+                                                                              mod$dimnames_terms),
+                                              matrix_covariates = matrix(NA_real_, nrow = 0, ncol = 0))
+  expect_equal(ans_obtained, ans_expected)
+})
+
+test_that("'make_outcome_offset_matrices' works with model with numeric covariates", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = 1:2,
+                      sex = c("F", "M"),
+                      time = 1:2)
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age * sex + region
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn) |>
+    set_covariates(~ income)
+  ans_obtained <- make_outcome_offset_matrices(mod, aggregate = TRUE)
+  data_ag <- make_data_df(mod)
+  outcome_df <- aggregate(data_ag["deaths"], data_ag[c("age", "sex", "region", "income")], sum)
+  offset_df <- aggregate(data_ag["popn"], data_ag[c("age", "sex", "region", "income")], sum)
+  data_ag <- merge(outcome_df, offset_df, by = c("age", "sex", "region", "income"))
+  ans_expected <- list(outcome = data_ag[["deaths"]],
+                       offset = data_ag[["popn"]],
+                       matrices_effect_outcome = make_matrices_effect_outcome(data_ag,
+                                                                              mod$dimnames_terms),
+                       matrix_covariates = matrix(scale(data_ag$income),
+                                                  nrow = nrow(data),
+                                                  dimnames = list(NULL, "income")))
+  expect_equal(ans_obtained, ans_expected)
+})
+
+test_that("'make_outcome_offset_matrices' works with model with categorical covariates", {
+  set.seed(0)
+  data <- expand.grid(age = 0:9,
+                      region = 1:2,
+                      sex = c("F", "M"),
+                      time = as.character(1:3))
+  data$popn <- rpois(n = nrow(data), lambda = 100)
+  data$deaths <- rpois(n = nrow(data), lambda = 10)
+  data$income <- rnorm(n = nrow(data))
+  formula <- deaths ~ age * sex + region
+  mod <- mod_pois(formula = formula,
+                  data = data,
+                  exposure = popn) |>
+    set_covariates(~ time)
+  ans_obtained <- make_outcome_offset_matrices(mod, aggregate = TRUE)
+  data_ag <- make_data_df(mod)
+  outcome_df <- aggregate(data_ag["deaths"], data_ag[c("age", "sex", "region", "time")], sum)
+  offset_df <- aggregate(data_ag["popn"], data_ag[c("age", "sex", "region", "time")], sum)
+  data_ag <- merge(outcome_df, offset_df, by = c("age", "sex", "region", "time"))
+  ans_expected <- list(outcome = data_ag[["deaths"]],
+                       offset = data_ag[["popn"]],
+                       matrices_effect_outcome = make_matrices_effect_outcome(data_ag,
+                                                                              mod$dimnames_terms),
+                       matrix_covariates = cbind(time2 = rep(c(0L, 1L, 0L), times = 40),
+                                                 time3 = rep(c(0L, 0L, 1L), times = 40)))
+  expect_equal(ans_obtained, ans_expected)
+})
+
+
 ## 'make_prior_class' ---------------------------------------------------------
 
 test_that("'make_prior_class' works with valid inputs", {
@@ -1229,24 +1511,6 @@ test_that("'make_priors' works with valid inputs - has intercept", {
 })
 
 
-## 'make_random' --------------------------------------------------------------
-
-test_that("'make_random' works when no hyper, no hyperrandfree", {
-    mod <- structure(.Data = list(priors = list(NFix(), Known(c(2, 3)))))
-    expect_identical(make_random(mod), NULL)
-})
-
-test_that("'make_random' works when hyper, no hyperrandfree", {
-    mod <- structure(.Data = list(priors = list(N(), RW2())))
-    expect_identical(make_random(mod), "effectfree")
-})
-
-test_that("'make_random' works when hyper, hyperrand", {
-    mod <- structure(.Data = list(priors = list(N(), RW2(), Lin())))
-    expect_identical(make_random(mod), c("effectfree", "hyperrandfree"))
-})
-
-
 ## 'make_seed' --------------------------------------------------------------
 
 test_that("'make_seed' returns a single unique integer", {
@@ -1510,7 +1774,7 @@ test_that("'make_uses_matrix_effectfree_effect' works with valid inputs", {
 })
 
 
-## 'make_uses_matrix_effectfree_effect' ---------------------------------------------
+## 'make_uses_matrix_effectfree_effect' ---------------------------------------
 
 test_that("'make_uses_offset_effectfree_effect' works with valid inputs", {
     set.seed(0)
@@ -1534,95 +1798,6 @@ test_that("'make_uses_offset_effectfree_effect' works with valid inputs", {
 })
 
 
-## 'make_vals_ag' -------------------------------------------------------------
-
-test_that("'make_vals_ag' works with model with offset", {
-  set.seed(0)
-  data <- expand.grid(age = 0:9,
-                      region = 1:2,
-                      sex = c("F", "M"),
-                      time = 1:2)
-  data$popn <- rpois(n = nrow(data), lambda = 100)
-  data$deaths <- rpois(n = nrow(data), lambda = 10)
-  formula <- deaths ~ age * sex + region
-  mod <- mod_pois(formula = formula,
-                  data = data,
-                  exposure = popn)
-  ans_obtained <- make_vals_ag(mod)
-  data_ag <- aggregate(data[c("deaths", "popn")], data[c("age", "region", "sex")], sum)
-  data_ag <- data_ag[with(data_ag, order(age, sex, region)), ]
-  ans_expected <- list(outcome = data_ag[["deaths"]],
-                       offset = data_ag[["popn"]],
-                       matrices_effect_outcome = make_matrices_effect_outcome(data_ag,
-                                                                              mod$dimnames_terms))
-  expect_equal(ans_obtained, ans_expected)
-})
-
-test_that("'make_vals_ag' works with model without offset", {
-  set.seed(0)
-  data <- expand.grid(age = 0:9,
-                      region = 1:2,
-                      sex = c("F", "M"),
-                      time = 1:2)
-  data$deaths <- rpois(n = nrow(data), lambda = 10)
-  formula <- deaths ~ age * sex + region
-  mod <- mod_pois(formula = formula,
-                  data = data,
-                  exposure = 1)
-  ans_obtained <- make_vals_ag(mod)
-  data_ag <- aggregate(data["deaths"], data[c("age", "sex", "region")], sum)
-  ans_expected <- list(outcome = data_ag[["deaths"]],
-                       offset = rep(1, times = nrow(data_ag)),
-                       matrices_effect_outcome = make_matrices_effect_outcome(data_ag,
-                                                                              mod$dimnames_terms))
-  expect_equal(ans_obtained, ans_expected)
-})
-
-
-## 'make_vals_in_lik' -------------------------------------------------------------
-
-test_that("'make_vals_in_lik' works with model with offset", {
-  set.seed(0)
-  data <- expand.grid(age = 0:9,
-                      region = 1:2,
-                      sex = c("F", "M"),
-                      time = 1:2)
-  data$popn <- rpois(n = nrow(data), lambda = 100)
-  data$deaths <- rpois(n = nrow(data), lambda = 10)
-  data$popn[1] <- 0
-  data$deaths[1] <- 0
-  formula <- deaths ~ age * sex + region
-  mod <- mod_pois(formula = formula,
-                  data = data,
-                  exposure = popn)
-  ans_obtained <- make_vals_in_lik(mod)
-  ans_expected <- list(outcome = mod$outcome[-1],
-                       offset = mod$offset[-1],
-                       matrices_effect_outcome = make_matrices_effect_outcome(data[-1,],
-                                                                              mod$dimnames_terms))
-  expect_equal(ans_obtained, ans_expected)
-})
-
-test_that("'make_vals_in_lik' works with model with offset", {
-  set.seed(0)
-  data <- expand.grid(age = 0:9,
-                      region = 1:2,
-                      sex = c("F", "M"),
-                      time = 1:2)
-  data$deaths <- rpois(n = nrow(data), lambda = 10)
-  data$deaths[1] <- NA
-  formula <- deaths ~ age * sex + region
-  mod <- mod_pois(formula = formula,
-                  data = data,
-                  exposure = 1)
-  ans_obtained <- make_vals_in_lik(mod)
-  ans_expected <- list(outcome = mod$outcome[-1],
-                       offset = mod$offset[-1],
-                       matrices_effect_outcome = make_matrices_effect_outcome(data[-1,],
-                                                                              mod$dimnames_terms))
-  expect_equal(ans_obtained, ans_expected)
-})
-
 
 ## 'make_vars_inner' ----------------------------------------------------------
 
@@ -1679,6 +1854,53 @@ test_that("'make_vars_inner' throws correct error with age, sex, time not presen
 })
 
 
+## 'message_suspicious_rates' -------------------------------------------------
+
+test_that("'message_suspicious_rates' returns NULL with valid inputs", {
+  outcome <- c(0:40, NA)
+  exposure <- c(0.000001, rep(3, 41))
+  ans_obtained <- message_suspicious_rates(outcome = outcome,
+                                           exposure = exposure,
+                                           mult_high_rate = 1000)
+  ans_expected <- NULL
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'message_suspicious_rates' returns NULL with less than 20 obs", {
+  outcome <- 1
+  exposure <- 0.00000001
+  ans_obtained <- message_suspicious_rates(outcome = outcome,
+                                           exposure = exposure,
+                                           mult_high_rate = 1000)
+  ans_expected <- NULL
+  expect_identical(ans_obtained, ans_expected)
+})
+
+test_that("'message_suspicious_rates' returns message with 1 row", {
+  outcome <- rep(1, 21)
+  exposure <- c(rep(10, 20), 0.00000001)
+  suppressMessages(
+    expect_message(message_suspicious_rates(outcome = outcome,
+                                            exposure = exposure,
+                                            mult_high_rate = 1000),
+                   "`data` has row with unexpectedly high rate.")
+  )
+})
+
+test_that("'message_suspicious_rates' returns message with 2 rows", {
+  outcome <- rep(1, 43)
+  exposure <- c(rep(10, 40), NA, 0.00000001, 0.0000002)
+  suppressMessages(
+    expect_message(message_suspicious_rates(outcome = outcome,
+                                            exposure = exposure,
+                                            mult_high_rate = 100),
+                   "`data` has rows with unexpectedly high rates.")
+  )
+})
+
+
+
+
 ## 'n_col' --------------------------------------------------------------------
 
 test_that("'n_col' works with ordinary matrix", {
